/*
  Project: NEO-6M GY-GPS6MV2 GPS module, LCD 2004 I2C module
  Function: This sketch listen to the GPS serial port,
  and when data received from the module, it's displays date, time, position on LCD 2004 I2C module.

  Wiring GPS module:
  GPS module -> Arduino Uno
  GND           GND
  RX pin        Digital pin 3
  TX pin        Digital pin 4
  VCC pin       5V

  Wiring LCD 2004 I2C module:
 LCD2004 I2C module -> Arduino Uno
  VCC           5V 
  SDA           SDA
  SCL           SCL
  GND           GND
 
*/
#include <SoftwareSerial.h>//include library code to allow serial communication on other digital pins of the Arduino board
#include <TinyGPS++.h>//include the library code for GPS module
#include <LiquidCrystal_I2C.h>//include the library code for LCD module

//Constants defined here
static const int RXPin = 4, TXPin = 3;
static const uint32_t GPSBaud = 9600;
byte last_second;
char Time[]  = "TIME: 00:00:00";
char Date[]  = "DATE: 00/00/2000";
//Initialise the LCD 2004 I2C
LiquidCrystal_I2C lcd(0x3F, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);
//The TinyGPS++ object
TinyGPSPlus gps;
//The serial connection to the GPS device
SoftwareSerial ss(RXPin, TXPin);

void setup()
{
  //initialise serial ciommunication at 115200 bps
  Serial.begin(115200);
  //define the LCD as 20 column by 4 rows
  lcd.begin (20, 4);
  //switch on the backlight
  lcd.setBacklightPin(3, POSITIVE);
  lcd.setBacklight(HIGH);
  //goto first column (column 0) and first line (Line 0)
  lcd.setCursor(0, 0);
  //Print at cursor Location
  lcd.print(" Acoptex GPS-Timing");
  //goto first column (column 0) and second line (line 1)
  lcd.setCursor(0, 1);
  lcd.print("More DIY projects on");
  //goto first column (column 0) and second line (line 2)
  lcd.setCursor(0, 2);
  lcd.print(" http://acoptex.com");
  ss.begin(GPSBaud);
  delay(2000);
  //clears the screen
  lcd.clear();
}
void loop()
{
  //displays information every time a new sentence is correctly encoded.
  while (ss.available() > 0)
    if (gps.encode(ss.read()))
      displayInfo(); 
  if (millis() > 5000 && gps.charsProcessed() < 10)
  {

    lcd.setCursor(3, 0);
    lcd.print("No GPS detected:");
    lcd.setCursor(3, 1);
    lcd.print(" check wiring.");
    lcd.setCursor(2, 2);
    lcd.print("More projects are");
    lcd.setCursor(3, 3);
    lcd.print(" on Acoptex.com");
    while (true);
  }
  delay(100);
}

void displayInfo()
{
  if (gps.date.isValid())
  {
    //goto first column (column 0) and first line (Line 0)
    lcd.setCursor(0, 0);
    Date[6]  = gps.date.day()    / 10 + 48;
    Date[7]  = gps.date.day()    % 10 + 48;
    Date[9]  = gps.date.month()  / 10 + 48;
    Date[10]  = gps.date.month()  % 10 + 48;
    Date[14] = (gps.date.year()   / 10) % 10 + 48;
    Date[15] = gps.date.year()   % 10 + 48;
    //Print at cursor Location
    lcd.print(Date);
  }
  else
  {
    lcd.setCursor(0, 0);
    lcd.print("DATE: INVALID");
  }
  if (gps.time.isValid())
  {
    Time[6]  = gps.time.hour()   / 10 + 48;
    Time[7]  = gps.time.hour()   % 10 + 48;
    Time[9]  = gps.time.minute() / 10 + 48;
    Time[10]  = gps.time.minute() % 10 + 48;
    Time[12] = gps.time.second() / 10 + 48;
    Time[13] = gps.time.second() % 10 + 48;
    //goto first column (column 0) and second line (line 1)
    lcd.setCursor(0, 1);
    //Print at cursor Location
    lcd.print(Time);
  }
  else
  {
    lcd.setCursor(0, 1);
    lcd.print("TIME: INVALID");
  }
  if (gps.location.isValid())
  {
    lcd.setCursor(0, 2);
    lcd.print("LATITUDE:  ");
    lcd.setCursor(11, 2);
    lcd.print(gps.location.lat(), 6);
    lcd.setCursor(0, 3);
    lcd.print("LONGITUDE: ");
    lcd.setCursor(11, 3);
    lcd.print(gps.location.lng(), 6);
  }
  else
  {
    lcd.setCursor(0, 2);
    lcd.print("LATITUDE:  INVALID");
    lcd.setCursor(0, 3);
    lcd.print("LONGITUDE: INVALID");
  }  
}
